package xmllecture;

import com.sun.developers.codesamples.dom.doc.DOMUtil;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException; 

/**
 * Some DOM Examples for XML Lecture.
 * 
 * @author bhayes
 */
public class DomExamples {

	/**
	 * Simple driver for DOM coding examples.
	 * @param args command line arguments
	 */
	public static void main(String[] args) {
		DomExamples de = new DomExamples();
		System.out.println(de.getClass().getName() + ".main: Start.");
		Document doc1, doc2;
		System.out.println("--- createSimpleDocument --- ");
		doc1 = de.createSimpleDocument();
		System.out.println("--- print(doc) --- ");
		de.print(doc1);
		System.out.println("--- readXmlDocument ---");
		//doc2 = de.readXmlDocument("mascots.xml");
		doc2 = DOMUtil.parse("mascots.xml");
		System.out.println("--- print(doc) --- ");
		de.print(doc2);
		DOMUtil.writeXmlToFile("temp.xml", doc2);
		System.out.println(de.getClass().getName() + ".main: Exit.");
	}
	/**
	 * Creates and returns a simple DOM document. That looks like this:
	 * <pre>
	 *    <Mascot>Oski</Mascot>
	 * </pre>
	 * @returns a document or null
	 */
	private Document createSimpleDocument() {
		Document doc = null;
		Text textNode;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder builder = factory.newDocumentBuilder();
			// Create a document
			doc = builder.newDocument();
			// Create a root element and add to document
			Element root = doc.createElement("Mascot");
			doc.appendChild(root);
			// Create a child element for root element
			Element e = doc.createElement("Name");
			root.appendChild(e);
			// Give the child element some text
			textNode = doc.createTextNode("Oski");
			e.appendChild(textNode);
			// Give the root element some attributes
			root.setAttribute("id", "0001");
			Attr a = doc.createAttribute("type");
			a.setValue("bear");
			root.setAttributeNode(a);
		} catch (ParserConfigurationException pce) {
			// Exception will inidcate that a parser with specified options can't be built
			pce.printStackTrace();
		}
		return doc;
	} //createEmptyDocument
	/**
	 * Prints the node contents. This code is not perfect.
	 * <p>This code was adopted from "Java and the XML DOM", by Steven Holzner,
	 * http://www.topxml.com/xml/articles/java_xml_dom/default.asp.
	 * </p>
	 * @param node
	 */
	public StringBuffer nodeToString(Node node, String indent) {
		StringBuffer sb = new StringBuffer("");
		if (node == null) {
			return sb;
		}
		int type = node.getNodeType();
		switch (type) {
			case Node.DOCUMENT_NODE :
				{ //If its a document node, get and process the root element
					sb =
						nodeToString(
							((Document) node).getDocumentElement(),
							"");
					break;
				}
			case Node.ELEMENT_NODE :
				{
					Attr attribute;
					NamedNodeMap attrs;
					// 1. Get element name
					//sb.append(indent + "<" + node.getNodeName());
					Element e = (Element) node;
					sb.append(indent + "<" + e.getTagName());
					// 2. Get the collection of attributes 
					attrs = node.getAttributes();
					for (int i = 0; i < attrs.getLength(); i++) {
						attribute = (Attr) attrs.item(i);
						sb.append(
							" "
								+ attribute.getNodeName()
								+ "=\""
								+ attribute.getNodeValue()
								+ "\"");
					}
					// 3.  Process the child nodes, if any
					NodeList childNodes = node.getChildNodes();
					if (childNodes == null) {
						sb.append("/>");
					} else if (childNodes.getLength() == 0) {
						sb.append("/>");
					} else {
						sb.append(">\n");
						for (int i = 0; i < childNodes.getLength(); i++) {
							sb.append(
								nodeToString(
									childNodes.item(i),
									indent + "   "));
						}
						sb.append("\n");
						sb.append (indent + "</" + node.getNodeName() + ">");
					}
					break;
				}
			case Node.TEXT_NODE :
				{
					sb.append(indent + node.getNodeValue().trim());
					break;
				}

		} // switch
		return sb;
	} //nodeToString
	
	/**
	 * @param doc
	 */
	private void print(Document doc) {
		System.out.println(nodeToString((Node) doc, ""));
	}
	/**
	 * Reads an XML from the indicated file and returns 
	 * a DOM document version of the XML.
	 * <p>
	 * Copied from DomEcho01.java, Copyright (c) 2002 Sun Microsystems, Inc.
	 * </p>
	 * @param filepath
	 * @return
	 */
	private Document readXmlDocument(String file) {
		Document document = null;
		DocumentBuilderFactory factory =
			DocumentBuilderFactory.newInstance();
		//factory.setValidating(true);   
		//factory.setNamespaceAware(true);
		try {
		   DocumentBuilder builder = factory.newDocumentBuilder();
		   document = builder.parse(file);

		} catch (SAXParseException spe) {
		   // Error generated by the parser
		   System.out.println("\n** Parsing error"
			  + ", line " + spe.getLineNumber()
			  + ", uri " + spe.getSystemId());
		   System.out.println("   " + spe.getMessage() );

		   // Use the contained exception, if any
		   Exception  x = spe;
		   if (spe.getException() != null)
			   x = spe.getException();
		   x.printStackTrace();
 
		} catch (SAXException sxe) {
		   // Error generated during parsing)
		   Exception  x = sxe;
		   if (sxe.getException() != null)
			   x = sxe.getException();
		   x.printStackTrace();

		} catch (ParserConfigurationException pce) {
			// Parser with specified options can't be built
			pce.printStackTrace();

		} catch (IOException ioe) {
		   // I/O error
		   ioe.printStackTrace();
		}

		return document;
	} //readXmlDocument
}
